﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win.UltraWinDock;
using System.IO;
using System.Xml;
using System.Drawing;
using System.Windows.Forms;
using System.Xml.Linq;

namespace WinDockManagerSaveLayout
{
    static class XMLWrapper
    {
        public static void Load(UltraDockManager ultraDockManager, FileStream stream)
        {
            string Path = stream.Name;
            stream.Close();

            XmlDocument xml = new XmlDocument();
            xml.Load(Path);
            XmlNodeList elemList = xml.GetElementsByTagName("PrimaryMonitorSize");
            if (elemList.Count == 0)
            {
                ultraDockManager.LoadFromXML(Path);
                return;
            }

            int width = -1;
            int.TryParse(elemList[0].ChildNodes[0].ChildNodes[0].Value, out width);
            int height = -1;
            int.TryParse(elemList[0].ChildNodes[1].ChildNodes[0].Value, out height);
            xml.ChildNodes[0].RemoveChild(elemList[0]);

            Size SavedMonitorSize = new Size(width, height);
            Size MonitorSize = SystemInformation.PrimaryMonitorSize;
            
            if (SavedMonitorSize != MonitorSize)
            {
               ProccessXML(ref xml, SavedMonitorSize);
            }
            xml.Save(Path);
            ultraDockManager.LoadFromXML(Path);
            xml.ChildNodes[0].AppendChild(CreatePrimaryMonitorSizeNode(xml));
            xml.Save(Path);
        }

        public static void Save(UltraDockManager ultraDockManager, FileStream stream)
        {
            string Path = stream.Name;
            stream.Close();

            ultraDockManager.SaveAsXML(Path);

            XmlDocument xml = new XmlDocument();
            xml.Load(Path);
            XmlNodeList elemList = xml.GetElementsByTagName("PrimaryMonitorSize");
            if (elemList.Count == 0)
            {
                xml.ChildNodes[0].AppendChild(CreatePrimaryMonitorSizeNode(xml));
                elemList = xml.GetElementsByTagName("PrimaryMonitorSize");
            }

            elemList[0].ChildNodes[0].ChildNodes[0].Value = SystemInformation.PrimaryMonitorSize.Width.ToString();
            elemList[0].ChildNodes[1].ChildNodes[0].Value = SystemInformation.PrimaryMonitorSize.Height.ToString();
            xml.Save(Path);

        }

        private static XmlNode CreatePrimaryMonitorSizeNode(XmlDocument xmlDoc)
        {
            XmlElement el = xmlDoc.CreateElement("PrimaryMonitorSize");
            el.InnerXml = string.Format(@"<PrimaryMonitorSize><width>{0}</width><height>{1}</height></PrimaryMonitorSize>",
                SystemInformation.PrimaryMonitorSize.Width, SystemInformation.PrimaryMonitorSize.Height);
            return el.ChildNodes[0];
        }

        private static void ProccessXML(ref XmlDocument xmlDoc, Size SavedMonitorSize)
        {
            XmlNodeList elemList = xmlDoc.GetElementsByTagName("Size");
            Size MonitorSize = SystemInformation.PrimaryMonitorSize;
            foreach (XmlNode node in elemList)
            {
                int width = -1;int height = -1;
                int.TryParse(node.ChildNodes[0].ChildNodes[0].Value, out width);
                int.TryParse(node.ChildNodes[1].ChildNodes[0].Value, out height);
                double wPercent = (double)width / (double)SavedMonitorSize.Width;
                double hPercent = (double)height / (double)SavedMonitorSize.Height;
                node.ChildNodes[0].ChildNodes[0].Value = ((int)Math.Ceiling(MonitorSize.Width * wPercent)).ToString();
                node.ChildNodes[1].ChildNodes[0].Value = ((int)Math.Ceiling(MonitorSize.Height * hPercent)).ToString();
            }
        }

    }
}
